using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class EvilValidatorTests
    {
        // --------------------------------------------------------------------------
        // There isn't anything special about these tests, other than they were taken
        // from another test class to easily test the EvilValidator class. We could
        // duplicate all tests in all the other test classes for the sake of testing
        // EvilValidator, but there's not much point.
        //
        // And notice that the private TestEntity class used for these tests does not
        // inherit from EvilBaseEntity nor does it implement IValidatable. It's just a
        // plain old object that uses the attributes.
        // --------------------------------------------------------------------------

        [TestMethod()]
        public void EvilValidator_Date1IsOnDate2Test()
        {
            TestEntity entity = new TestEntity();
            entity.DateTime1 = DateTime.Today;
            entity.DateTime2 = DateTime.Today;

            EvilValidator ev = new EvilValidator(entity);

            Assert.IsFalse(ev.IsValid(), "Entity passed validation.");
            Assert.IsTrue(ev.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void EvilValidator_Date1IsBeforeDate2Test()
        {
            TestEntity entity = new TestEntity();
            entity.DateTime1 = DateTime.Today.AddDays(-1);
            entity.DateTime2 = DateTime.Today;

            EvilValidator ev = new EvilValidator(entity);

            Assert.IsFalse(ev.IsValid(), "Entity passed validation.");
            Assert.IsTrue(ev.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void EvilValidator_Date1IsAfterDate2Test()
        {
            TestEntity entity = new TestEntity();
            entity.DateTime1 = DateTime.Today.AddDays(1);
            entity.DateTime2 = DateTime.Today;

            EvilValidator ev = new EvilValidator(entity);

            Assert.IsTrue(ev.IsValid(), "Entity failed validation.");
            Assert.IsTrue(ev.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        #region Private TestEntity Class

        private class TestEntity
        {
            private DateTime _dateTime1;
            private DateTime _dateTime2;

            [ValidateDateOccursAfter("DateTime2", "DateTime1 must be after DateTime2.")]
            public DateTime DateTime1
            {
                get { return _dateTime1; }
                set { _dateTime1 = value; }
            }

            [ValidateDateOccursBefore("DateTime1", "DateTime2 must be before DateTime1.")]
            public DateTime DateTime2
            {
                get { return _dateTime2; }
                set { _dateTime2 = value; }
            }
        }

        #endregion
    }
}
