using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class FieldTests
    {
        // --------------------------------------------------------------------------
        // There isn't anything special about these tests, other than they are used
        // to test public and private fields (instead of actual properties like all
        // the other tests are doing). We could duplicate all tests in all the other
        // test classes for the sake of testing public and private fields, but there's
        // not much point.
        // --------------------------------------------------------------------------

        [TestMethod()]
        public void PublicField_Amount1IsLessThanAmount2Test()
        {
            TestEntityWithPublicFields entity = new TestEntityWithPublicFields();
            entity.Amount1 = 5;
            entity.Amount2 = 6;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void PublicField_Amount1IsGreaterThanAmount2Test()
        {
            TestEntityWithPublicFields entity = new TestEntityWithPublicFields();
            entity.Amount1 = 6;
            entity.Amount2 = 5;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void PublicField_Amount1IsEqualToAmount2Test()
        {
            TestEntityWithPublicFields entity = new TestEntityWithPublicFields();
            entity.Amount1 = 5;
            entity.Amount2 = 5;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void PrivateField_Amount1IsLessThanAmount2Test()
        {
            TestEntityWithPrivateFields entity = new TestEntityWithPrivateFields(5, 6);

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void PrivateField_Amount1IsGreaterThanAmount2Test()
        {
            TestEntityWithPrivateFields entity = new TestEntityWithPrivateFields(6, 5);

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void PrivateField_Amount1IsEqualToAmount2Test()
        {
            TestEntityWithPrivateFields entity = new TestEntityWithPrivateFields(5, 5);

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        #region Private TestEntity Classes

        private class TestEntityWithPublicFields : EvilBaseEntity
        {
            [ValidateGreaterThan("Amount2", "Amount1 must be greater than Amount2.")]
            public int Amount1;

            [ValidateLessThan("Amount1", "Amount2 must be less than Amount1.")]
            public int Amount2;
        }

        private class TestEntityWithPrivateFields : EvilBaseEntity
        {
            public TestEntityWithPrivateFields(int amount1, int amount2)
            {
                _amount1 = amount1;
                _amount2 = amount2;
            }

            [ValidateGreaterThan("_amount2", "_amount1 must be greater than _amount2.")]
            private int _amount1;

            [ValidateLessThan("_amount1", "_amount2 must be less than _amount1.")]
            private int _amount2;
        }

        #endregion
    }
}