using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class PropertyFactoryTests
    {
        [TestMethod()]
        public void GetProperties_EntityWithPublicInstancePropertyTest()
        {
            int propValue = 1234;

            TestEntityPublicInstanceProperty entity = new TestEntityPublicInstanceProperty();
            entity.PublicInstanceProperty = propValue;

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "PublicInstanceProperty");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "PublicInstanceProperty", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithPublicStaticPropertyTest()
        {
            int propValue = 1234;

            TestEntityPublicStaticProperty entity = new TestEntityPublicStaticProperty();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "PublicStaticProperty");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "PublicStaticProperty", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithPrivateInstancePropertyTest()
        {
            int propValue = 1234;

            TestEntityPrivateInstanceProperty entity = new TestEntityPrivateInstanceProperty();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "PrivateInstanceProperty");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "PrivateInstanceProperty", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithPrivateStaticPropertyTest()
        {
            int propValue = 1234;

            TestEntityPrivateStaticProperty entity = new TestEntityPrivateStaticProperty();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "PrivateStaticProperty");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "PrivateStaticProperty", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithProtectedInstancePropertyTest()
        {
            int propValue = 1234;

            TestEntityProtectedInstanceProperty entity = new TestEntityProtectedInstanceProperty();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "ProtectedInstanceProperty");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "ProtectedInstanceProperty", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithProtectedStaticPropertyTest()
        {
            int propValue = 1234;

            TestEntityProtectedStaticProperty entity = new TestEntityProtectedStaticProperty();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "ProtectedStaticProperty");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "ProtectedStaticProperty", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithInternalInstancePropertyTest()
        {
            int propValue = 1234;

            TestEntityInternalInstanceProperty entity = new TestEntityInternalInstanceProperty();
            entity.InternalInstanceProperty = propValue;

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "InternalInstanceProperty");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "InternalInstanceProperty", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithInternalStaticPropertyTest()
        {
            int propValue = 1234;

            TestEntityInternalStaticProperty entity = new TestEntityInternalStaticProperty();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "InternalStaticProperty");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "InternalStaticProperty", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithPublicInstanceFieldTest()
        {
            int propValue = 1234;

            TestEntityPublicInstanceField entity = new TestEntityPublicInstanceField();
            entity.PublicInstanceField = propValue;

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "PublicInstanceField");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "PublicInstanceField", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithPublicStaticFieldTest()
        {
            int propValue = 1234;

            TestEntityPublicStaticField entity = new TestEntityPublicStaticField();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "PublicStaticField");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "PublicStaticField", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithProtectedInstanceFieldTest()
        {
            int propValue = 1234;

            TestEntityProtectedInstanceField entity = new TestEntityProtectedInstanceField();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "ProtectedInstanceField");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "ProtectedInstanceField", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithProtectedStaticFieldTest()
        {
            int propValue = 1234;

            TestEntityProtectedStaticField entity = new TestEntityProtectedStaticField();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "ProtectedStaticField");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "ProtectedStaticField", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithInternalInstanceFieldTest()
        {
            int propValue = 1234;

            TestEntityInternalInstanceField entity = new TestEntityInternalInstanceField();
            entity.InternalInstanceField = propValue;

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "InternalInstanceField");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "InternalInstanceField", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithInternalStaticFieldTest()
        {
            int propValue = 1234;

            TestEntityInternalStaticField entity = new TestEntityInternalStaticField();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "InternalStaticField");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "InternalStaticField", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithPrivateInstanceFieldTest()
        {
            int propValue = 1234;

            TestEntityPrivateInstanceField entity = new TestEntityPrivateInstanceField();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "_privateInstanceField");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "_privateInstanceField", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        [TestMethod()]
        public void GetProperties_EntityWithPrivateStaticFieldTest()
        {
            int propValue = 1234;

            TestEntityPrivateStaticField entity = new TestEntityPrivateStaticField();

            IList<ISmartMemberInfo> memberInfos = PropertyFactory.GetProperties(entity);

            Assert.IsNotNull(memberInfos, "List of member infos was null.");
            Assert.IsTrue(memberInfos.Count >= 1, "List of member infos did not contain enough items.");

            ISmartMemberInfo info = FindPropertyNamed(memberInfos, "_privateStaticField");

            Assert.IsNotNull(info, "Member info was null.");
            Assert.AreEqual(info.Name, "_privateStaticField", "Expected name did not equal actual name.");
            Assert.AreEqual(typeof(int), info.PropertyType, "Expected type did not equal actual type.");
            Assert.AreEqual(propValue, Convert.ToInt32(info.GetValue(entity, null)), "Expected value did not equal actual value.");
        }

        private ISmartMemberInfo FindPropertyNamed(IList<ISmartMemberInfo> memberInfos, string name)
        {
            foreach (ISmartMemberInfo info in memberInfos)
            {
                if (info.Name == name)
                {
                    return info;
                }
            }
            return null;
        }

        #region Private Test Classes

        private class TestEntityPublicInstanceProperty : EvilBaseEntity
        {
            private int _publicInstanceProperty;

            public int PublicInstanceProperty
            {
                get { return _publicInstanceProperty; }
                set { _publicInstanceProperty = value; }
            }
        }

        private class TestEntityPublicStaticProperty : EvilBaseEntity
        {
            private static int _publicStaticProperty = 1234;

            public static int PublicStaticProperty
            {
                get { return _publicStaticProperty; }
            }
        }

        private class TestEntityPrivateInstanceProperty : EvilBaseEntity
        {
            private int PrivateInstanceProperty
            {
                get { return 1234; }
            }
        }

        private class TestEntityPrivateStaticProperty : EvilBaseEntity
        {
            private static int PrivateStaticProperty
            {
                get { return 1234; }
            }
        }

        private class TestEntityProtectedInstanceProperty : EvilBaseEntity
        {
            private int _protectedInstanceProperty = 1234;

            protected int ProtectedInstanceProperty
            {
                get { return _protectedInstanceProperty; }
                set { _protectedInstanceProperty = value; }
            }
        }

        private class TestEntityProtectedStaticProperty : EvilBaseEntity
        {
            private static int _protectedStaticProperty = 1234;

            protected static int ProtectedStaticProperty
            {
                get { return _protectedStaticProperty; }
            }
        }

        private class TestEntityInternalInstanceProperty : EvilBaseEntity
        {
            private int _internalInstanceProperty;

            internal int InternalInstanceProperty
            {
                get { return _internalInstanceProperty; }
                set { _internalInstanceProperty = value; }
            }
        }

        private class TestEntityInternalStaticProperty : EvilBaseEntity
        {
            private static int _internalStaticProperty = 1234;

            internal static int InternalStaticProperty
            {
                get { return _internalStaticProperty; }
            }
        }

        private class TestEntityPublicInstanceField : EvilBaseEntity
        {
            public int PublicInstanceField;
        }

        private class TestEntityPublicStaticField : EvilBaseEntity
        {
            public static int PublicStaticField = 1234;
        }

        private class TestEntityPrivateInstanceField : EvilBaseEntity
        {
            private int _privateInstanceField = 1234;
        }

        private class TestEntityPrivateStaticField : EvilBaseEntity
        {
            private static int _privateStaticField = 1234;
        }

        private class TestEntityProtectedInstanceField : EvilBaseEntity
        {
            protected int ProtectedInstanceField = 1234;
        }

        private class TestEntityProtectedStaticField : EvilBaseEntity
        {
            protected static int ProtectedStaticField = 1234;
        }

        private class TestEntityInternalInstanceField : EvilBaseEntity
        {
            internal int InternalInstanceField;
        }

        private class TestEntityInternalStaticField : EvilBaseEntity
        {
            internal static int InternalStaticField = 1234;
        }

        #endregion
    }
}
