using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateDateOccursAfterAttributeTests
    {
        [TestMethod()]
        public void ValidateDateOccursAfterAttribute_Date1IsOnDate2Test()
        {
            TestEntity entity = new TestEntity();
            entity.DateTime1 = DateTime.Today;
            entity.DateTime2 = DateTime.Today;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateDateOccursAfterAttribute_Date1IsBeforeDate2Test()
        {
            TestEntity entity = new TestEntity();
            entity.DateTime1 = DateTime.Today.AddDays(-1);
            entity.DateTime2 = DateTime.Today;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateDateOccursAfterAttribute_Date1IsAfterDate2Test()
        {
            TestEntity entity = new TestEntity();
            entity.DateTime1 = DateTime.Today.AddDays(1);
            entity.DateTime2 = DateTime.Today;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private DateTime _dateTime1;
            private DateTime _dateTime2;

            [ValidateDateOccursAfter("DateTime2", "DateTime1 must be after DateTime2.")]
            public DateTime DateTime1
            {
                get { return _dateTime1; }
                set { _dateTime1 = value; }
            }

            public DateTime DateTime2
            {
                get { return _dateTime2; }
                set { _dateTime2 = value; }
            }
        }

        #endregion
    }
}
