using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateMaxLengthAttributeTests
    {
        [TestMethod()]
        public void ValidateMaxLengthAttribute_PropertyEqualsMaxLengthTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "ABCDEFGHIJ";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateMaxLengthAttribute_PropertyIsLessThanMaxLengthTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "ABCDE";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateMaxLengthAttribute_PropertyIsNullTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = null;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateMaxLengthAttribute_PropertyIsEmptyTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = string.Empty;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateMaxLengthAttribute_PropertyIsGreaterThanMaxLengthTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "TheLengthOfThisStringIsGreaterThanMaxLength";

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private string _property1;

            [ValidateMaxLength(10, "Property1 has a max length of 10.")]
            public string Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }
        }

        #endregion
    }
}
