using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateMinCountAttributeTests
    {
        [TestMethod()]
        public void ValidateMinCountAttribute_CollectionIsNullTest()
        {
            TestEntity entity = new TestEntity();

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateMinCountAttribute_CollectionCountEqualsMinCountTest()
        {
            TestEntity entity = new TestEntity();
            entity.Collection1.Add("Item1");
            entity.Collection1.Add("Item2");
            entity.Collection1.Add("Item3");

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateMinCountAttribute_CollectionCountIsLessThanMinCountTest()
        {
            TestEntity entity = new TestEntity();
            entity.Collection1.Add("Item1");
            entity.Collection1.Add("Item2");

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateMinCountAttribute_CollectionCountIsGreaterThanMinCountTest()
        {
            TestEntity entity = new TestEntity();
            entity.Collection1.Add("Item1");
            entity.Collection1.Add("Item2");
            entity.Collection1.Add("Item3");
            entity.Collection1.Add("Item4");

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private List<string> _collection1;

            [ValidateMinCount(3, "Collection1 must contain at least 3 items.")]
            public List<string> Collection1
            {
                get
                {
                    if (_collection1 == null)
                    {
                        _collection1 = new List<string>();
                    }
                    return _collection1;
                }
            }
        }

        #endregion
    }
}