using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateMinLengthAttributeTests
    {
        [TestMethod()]
        public void ValidateMinLengthAttribute_PropertyEqualsMinLengthTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "ABCDE";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateMinLengthAttribute_PropertyIsGreaterThanMinLengthTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "ThisStringLengthIsGreaterThanMinLength";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateMinLengthAttribute_PropertyIsLessThanMinLengthTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "ABC";

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateMinLengthAttribute_PropertyIsNullTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = null;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateMinLengthAttribute_PropertyIsEmptyTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = string.Empty;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private string _property1;

            [ValidateMinLength(5, "Property1 has a minimum length of 5.")]
            public string Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }
        }

        #endregion
    }
}
