using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateMinValueAttributeTests
    {
        [TestMethod()]
        public void ValidateMinValueAttribute_PropertyEqualsMinValueTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = 100;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateMinValueAttribute_PropertyIsGreaterThanMinValueTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = 100.0000001;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateMinValueAttribute_PropertyIsLessThanMinValueTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = 99.99999;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private double _property1;

            [ValidateMinValue(100, "Property1 is has a min value of 100.")]
            public double Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }
        }

        #endregion
    }
}
