using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidatePositiveNumberAttributeTests
    {
        [TestMethod()]
        public void ValidatePositiveNumberAttribute_PropertyIsPositiveTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = 1;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidatePositiveNumberAttribute_PropertyIsNegativeTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = -1;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidatePositiveNumberAttribute_PropertyIsZeroTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = 1;
            entity.Property2 = 0;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private int _property1;
            private int _property2;

            [ValidatePositiveNumber("Property1 must be a positive number.")]
            public int Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }

            [ValidatePositiveNumber(true, "Property2 must be a positive number, including zero.")]
            public int Property2
            {
                get { return _property2; }
                set { _property2 = value; }
            }
        }

        #endregion
    }
}
