using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateRegexAttributeTests
    {
        [TestMethod()]
        public void ValidateRegexAttribute_ZipCodeMatchesRegexTest1()
        {
            TestEntityZipCode entity = new TestEntityZipCode();
            entity.ZipCode = "43210";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_ZipCodeMatchesRegexTest2()
        {
            TestEntityZipCode entity = new TestEntityZipCode();
            entity.ZipCode = "43210-1234";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_ZipCodeMatchesRegexTest3()
        {
            TestEntityZipCode entity = new TestEntityZipCode();
            entity.ZipCode = "432101234";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_ZipCodeDoesNotMatchRegexTest()
        {
            TestEntityZipCode entity = new TestEntityZipCode();
            entity.ZipCode = "1a-*(&_@#$C";

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_ZipCodeIsNullTest()
        {
            TestEntityZipCode entity = new TestEntityZipCode();
            entity.ZipCode = null;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_ZipCodeIsEmptyTest()
        {
            TestEntityZipCode entity = new TestEntityZipCode();
            entity.ZipCode = string.Empty;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_EmailMatchesRegexTest()
        {
            TestEntityEmail entity = new TestEntityEmail();
            entity.Email = "name@company.com";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_EmailDoesNotMatchRegexTest()
        {
            TestEntityEmail entity = new TestEntityEmail();
            entity.Email = "name@.com";

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_EmailIsNullTest()
        {
            TestEntityEmail entity = new TestEntityEmail();
            entity.Email = null;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_EmailIsEmptyTest()
        {
            TestEntityEmail entity = new TestEntityEmail();
            entity.Email = string.Empty;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_GuidMatchesRegexTest()
        {
            TestEntityGuid entity = new TestEntityGuid();
            entity.Guid = "5594a1a5-e2b7-4d2b-8fda-726554a811b7";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_GuidDoesNotMatchRegexTest()
        {
            TestEntityGuid entity = new TestEntityGuid();
            entity.Guid = "5594a1a5e2b74d2b8fda726554a811b7";

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_GuidIsNullTest()
        {
            TestEntityGuid entity = new TestEntityGuid();
            entity.Guid = null;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRegexAttribute_GuidIsEmptyTest()
        {
            TestEntityGuid entity = new TestEntityGuid();
            entity.Guid = string.Empty;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        #region Private TestEntity Class

        private class TestEntityZipCode : EvilBaseEntity
        {
            private const string zipCodePattern = @"^[0-9]{5}([- /]?[0-9]{4})?$";
            private string _zipCode;

            [ValidateRegex(zipCodePattern, "Zip code does not match regular expression.")]
            public string ZipCode
            {
                get { return _zipCode; }
                set { _zipCode = value; }
            }
        }

        private class TestEntityEmail : EvilBaseEntity
        {
            private const string emailPattern = @"^([0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*@([0-9a-zA-Z][-\w]*[0-9a-zA-Z]\.)+[a-zA-Z]{2,9})$";
            private string _email;

            [ValidateRegex(emailPattern, "Email does not match regular expression.")]
            public string Email
            {
                get { return _email; }
                set { _email = value; }
            }
        }

        private class TestEntityGuid : EvilBaseEntity
        {
            private const string guidPattern = @"^(\{){0,1}[0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}(\}){0,1}$";
            private string _guid;

            [ValidateRegex(guidPattern, "Guid does not match regular expression.")]
            public string Guid
            {
                get { return _guid; }
                set { _guid = value; }
            }
        }

        #endregion
    }
}
