using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateRequiredAttributeTests
    {
        [TestMethod()]
        public void ValidateRequiredAttribute_PropertyIsNotNullAndNotEmptyTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "TestValue1";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredAttribute_PropertyIsNullTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = null;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRequiredAttribute_PropertyIsEmptyTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = string.Empty;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private string _property1;

            [ValidateRequired("Property1 is required.")]
            public string Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }
        }

        #endregion
    }
}
