using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateRequiredIfNotNullOrEmptyAttributeTests
    {
        [TestMethod()]
        public void ValidateRequiredIfNotNullOrEmptyAttribute_Property1IsPopulatedProperty2IsPopulated()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "Value1";
            entity.Property2 = "Value2";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredIfNotNullOrEmptyAttribute_Property1IsNullProperty2IsPopulated()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = null;
            entity.Property2 = "Value2";

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRequiredIfNotNullOrEmptyAttribute_Property1IsEmptyProperty2IsPopulated()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = string.Empty;
            entity.Property2 = "Value2";

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRequiredIfNotNullOrEmptyAttribute_Property1PopulatedProperty2IsNull()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "Value1";
            entity.Property2 = null;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredIfNotNullOrEmptyAttribute_Property1PopulatedProperty2IsEmpty()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "Value1";
            entity.Property2 = string.Empty;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private string _property1;
            private string _property2;

            [ValidateRequiredIfNotNullOrEmpty("Property2", "Property1 is required if Property2 is populated.")]
            public string Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }

            public string Property2
            {
                get { return _property2; }
                set { _property2 = value; }
            }
        }

        #endregion
    }
}
