using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateRequiredNullOrEmptyIfEqualsAttributeTests
    {
        [TestMethod()]
        public void ValidateRequiredNullOrEmptyIfEqualsAttribute_Property1IsNullProperty2EqualsValueTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = null;
            entity.Property2 = "TestValue";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredNullOrEmptyIfEqualsAttribute_Property1IsEmptyProperty2EqualsValueTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = string.Empty;
            entity.Property2 = "TestValue";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredNullOrEmptyIfEqualsAttribute_Property1IsNullProperty2DoesNotEqualValueTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = null;
            entity.Property2 = "WrongValue";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredNullOrEmptyIfEqualsAttribute_Property1IsEmptyProperty2DoesNotEqualValueTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = string.Empty;
            entity.Property2 = "WrongValue";

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredNullOrEmptyIfEqualsAttribute_Property1IsPopulatedProperty2EqualsValueTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "SomeValue";
            entity.Property2 = "TestValue";

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        [TestMethod()]
        public void ValidateRequiredNullOrEmptyIfEqualsAttribute_Property1IsNullProperty2IsNullTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = null;
            entity.Property2 = null;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredNullOrEmptyIfEqualsAttribute_Property1IsNullProperty2IsEmptyTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = null;
            entity.Property2 = string.Empty;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredNullOrEmptyIfEqualsAttribute_Property1IsPopulatedProperty2IsNullTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "SomeValue";
            entity.Property2 = null;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateRequiredNullOrEmptyIfEqualsAttribute_Property1IsPopulatedProperty2IsEmptyTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = "SomeValue";
            entity.Property2 = string.Empty;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private string _property1;
            private string _property2;

            [ValidateRequiredNullOrEmptyIfEquals("Property2", "TestValue", "Property1 must be null or empty if Property2 equals 'TestValue'.")]
            public string Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }

            public string Property2
            {
                get { return _property2; }
                set { _property2 = value; }
            }
        }

        #endregion
    }
}
