using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EvilAttributes;

namespace EvilAttributes.Tests
{
    [TestClass()]
    public class ValidateWholeNumberAttributeTests
    {
        [TestMethod()]
        public void ValidateWholeNumberAttribute_PropertyIsWholeNumberTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property1 = 100;

            Assert.IsTrue(entity.IsValid(), "Entity failed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count == 0, "ValidationErrors was not empty.");
        }

        [TestMethod()]
        public void ValidateWholeNumberAttribute_PropertyIsNotWholeNumberTest()
        {
            TestEntity entity = new TestEntity();
            entity.Property2 = 100.54M;

            Assert.IsFalse(entity.IsValid(), "Entity passed validation.");
            Assert.IsTrue(entity.ValidationErrors.Count > 0, "ValidationErrors was empty.");
        }

        #region Private TestEntity Class

        private class TestEntity : EvilBaseEntity
        {
            private long _property1;
            private decimal _property2;

            [ValidateWholeNumber("Property1 must be a whole number.")]
            public long Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }

            [ValidateWholeNumber("Property2 must be a whole number.")]
            public decimal Property2
            {
                get { return _property2; }
                set { _property2 = value; }
            }
        }

        #endregion
    }
}
