using System;
using System.Reflection;
using System.Collections.Generic;

namespace EvilAttributes
{
    /// <summary>
    /// Abstract class that entity classes can inherit from if they need to be validated
    /// by any of the EViL attributes. This abstract class provides a base implementation
    /// for being validated.
    /// </summary>
    [Serializable()]
    public abstract class EvilBaseEntity : IValidatable
    {
        private EvilValidator _validator;

        /// <summary>
        /// Intializes a new instance of a class of type EvilBaseEntity.
        /// </summary>
        public EvilBaseEntity()
        {
            _validator = new EvilValidator(this);
        }

        /// <summary>
        /// Gets the collection of validation errors for the entity.
        /// </summary>
        public IList<string> ValidationErrors
        {
            get { return _validator.ValidationErrors; }
        }

        /// <summary>
        /// Determines if the entity is valid.
        /// </summary>
        /// <returns>True if the entity is valid, false otherwise.</returns>
        /// <remarks>
        /// An entity is valid only if all attributes applied to the entity pass validation.
        /// </remarks>
        public virtual bool IsValid()
        {
            return _validator.IsValid();
        }
    }
}
