using System.Collections.Generic;
using System.Reflection;

namespace EvilAttributes
{
    public static class PropertyFactory
    {
        /// <summary>
        /// Gets all the properties and fields on the entity.
        /// </summary>
        /// <param name="entity">The entity from which to retrieve the properties and fields.</param>
        /// <returns>An IList containing all properties and fields for the entity.</returns>
        /// <remarks>
        /// This method returns all properties and fields that are public or private and instance or static.
        /// </remarks>
        public static IList<ISmartMemberInfo> GetProperties(object entity)
        {
            IList<ISmartMemberInfo> results = new List<ISmartMemberInfo>();

            GetReflectedProperties(entity, results, BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static);

            GetReflectedFields(entity, results, BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static);

            return results;
        }

        private static void GetReflectedProperties(object entity, IList<ISmartMemberInfo> values, BindingFlags bindingAttr)
        {
            PropertyInfo[] properties = entity.GetType().GetProperties(bindingAttr);

            foreach (PropertyInfo pi in properties)
            {
                ISmartMemberInfo smi = new SmartPropertyInfo(pi);
                values.Add(smi);
            }
        }

        private static  void GetReflectedFields(object entity, IList<ISmartMemberInfo> values, BindingFlags bindingAttr)
        {
            FieldInfo[] fields = entity.GetType().GetFields(bindingAttr);

            foreach (FieldInfo fi in fields)
            {
                ISmartMemberInfo smi = new SmartFieldInfo(fi);
                values.Add(smi);
            }
        }
    }
}