using System;
using System.Collections;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a collection has a maximum count. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidateMaxCountAttribute : EvilBaseAttribute
    {
        private int _maxCount;

        /// <summary>
        /// Gets the maximum count of the collection.
        /// </summary>
        public int MaxCount
        {
            get { return _maxCount; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateMaxCountAttribute class.
        /// </summary>
        /// <param name="maxCount">The maximum count of the collection.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateMaxCountAttribute(int maxCount, string message)
            : base(message)
		{
			_maxCount = maxCount;
		}

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
		{
            IList coll = value as IList;

            if (value != null)
            {
                if (coll.Count <= this.MaxCount)
                {
                    return true;
                }
            }

            // Otherwise
            return false;
		}
    }
}
