using System;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property has a maximum length. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidateMaxLengthAttribute : EvilBaseAttribute
    {
        private int _maxLength;

        /// <summary>
        /// Gets the maximum length of the property.
        /// </summary>
        public int MaxLength
        {
            get { return _maxLength; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateMaxLengthAttribute class.
        /// </summary>
        /// <param name="maxLength">The maximum length of the property.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateMaxLengthAttribute(int maxLength, string message)
            : base(message)
		{
			_maxLength = maxLength;
		}

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
		{
            if (value == null)
            {
                return true;
            }

            if (value != null)
            {
                if (value.ToString().Length <= this.MaxLength)
                {
                    return true;
                }
            }

            // Otherwise
            return false;
		}
    }
}
