using System;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property has a maximum value. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidateMaxValueAttribute : EvilBaseAttribute
    {
        private double _maxValue;

        /// <summary>
        /// Gets the maximum value of the property.
        /// </summary>
        public double MaxValue
        {
            get { return _maxValue; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateMaxValueAttribute class.
        /// </summary>
        /// <param name="maxValue">The maximum value of the property.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateMaxValueAttribute(double maxValue, string message)
            : base(message)
		{
			_maxValue = maxValue;
		}

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
		{
            double inputValue = Convert.ToDouble(value, null);

            if (inputValue <= this.MaxValue)
            {
                return true;
            }

            // Otherwise
            return false;
		}
    }
}
