using System;
using System.Collections;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a collection has a minimum count. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidateMinCountAttribute : EvilBaseAttribute
    {
        private int _minCount;

        /// <summary>
        /// Gets the minimum count of the collection.
        /// </summary>
        public int MinCount
        {
            get { return _minCount; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateMinCountAttribute class.
        /// </summary>
        /// <param name="minCount">The minimum count of the collection.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateMinCountAttribute(int minCount, string message)
            : base(message)
		{
			_minCount = minCount;
		}

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
		{
            IList coll = value as IList;

            if (value != null)
            {
                if (coll.Count >= this.MinCount)
                {
                    return true;
                }
            }

            // Otherwise
            return false;
		}
    }
}
