using System;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property has a minimum length. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidateMinLengthAttribute : EvilBaseAttribute
    {
        private int _minLength;

        /// <summary>
        /// Gets the minimum length of the property.
        /// </summary>
        public int MinLength
        {
            get { return _minLength; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateMinLengthAttribute class.
        /// </summary>
        /// <param name="minLength">The minimum length of the property.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateMinLengthAttribute(int minLength, string message)
            : base(message)
		{
			_minLength = minLength;
		}

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
		{
            if (value != null)
            {
                if (value.ToString().Length >= this.MinLength)
                {
                    return true;
                }
            }

            // Otherwise
            return false;
		}
    }
}
