using System;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property has a minimum value. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidateMinValueAttribute : EvilBaseAttribute
    {
        private double _minValue;

        /// <summary>
        /// Gets the minimum value of the property.
        /// </summary>
        public double MinValue
        {
            get { return _minValue; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateMinValueAttribute class.
        /// </summary>
        /// <param name="minValue">The minimum value of the property.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateMinValueAttribute(double minValue, string message)
            : base(message)
		{
			_minValue = minValue;
		}

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
		{
            double inputValue = Convert.ToDouble(value, null);

            if (inputValue >= this.MinValue)
            {
                return true;
            }

            // Otherwise
            return false;
		}
    }
}
