using System;
using System.Reflection;    

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property must be a multiple of another property. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]
    public sealed class ValidateMultipleOfAttribute : EvilBaseAttribute
    {
        private string _propertyName;

        /// <summary>
        /// Gets the name of the property to compare.
        /// </summary>
        public string PropertyName
        {
            get { return _propertyName; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateMultipleOfAttribute class.
        /// </summary>
        /// <param name="propertyName">The name of the property to compare.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateMultipleOfAttribute(string propertyName, string message)
            : base(message)
        {
            _propertyName = propertyName;
        }

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
        {

            long propValue = Convert.ToInt64(base.GetPropertyValue(this.PropertyName, entity), null);
            long inputValue = Convert.ToInt64(value, null);

            if (inputValue % propValue == 0)
            {
                return true;
            }

            // Otherwise
            return false;
        }
    }
}
