using System;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property must be a positive number. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidatePositiveNumberAttribute : EvilBaseAttribute
    {
        private bool _includeZero;

        /// <summary>
        /// Gets the value indicating whether or not to include zero in the validation.
        /// </summary>
        public bool IncludeZero
        {
            get { return _includeZero; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidatePositiveNumberAttribute class.
        /// </summary>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidatePositiveNumberAttribute(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the ValidatePositiveNumberAttribute class.
        /// </summary>
        /// <param name="includeZero">Determines if zero should be included in the validation.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidatePositiveNumberAttribute(bool includeZero, string message)
            : base(message)
        {
            _includeZero = includeZero;
        }

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
        {
            bool result = false;
            decimal convertedVal = Convert.ToDecimal(value, null);

            if (this.IncludeZero)
            {
                if (convertedVal >= 0)
                {
                    result = true;
                }
            }
            else
            {
                if (convertedVal > 0)
                {
                    result = true;
                }
            }

            return result;
        }
    }
}
