using System;
using System.Text.RegularExpressions;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property must match a regular expression pattern. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidateRegexAttribute : EvilBaseAttribute
    {
        private string _pattern;

        /// <summary>
        /// Gets the regular expression pattern used to validate.
        /// </summary>
        public string Pattern
        {
            get { return _pattern; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateRegexAttribute class.
        /// </summary>
        /// <param name="pattern">The regular expression pattern used to validate.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateRegexAttribute(string pattern, string message)
            : base(message)
        {
            _pattern = pattern;
        }

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
        {
            if (value == null || value.ToString().Length == 0)
            {
                return false;
            }

            Regex re = new Regex(this.Pattern);
            return re.IsMatch(value.ToString());
        }
    }
}
