using System;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property is required. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidateRequiredAttribute : EvilBaseAttribute
    {
        /// <summary>
        /// Initializes a new instance of the ValidateRequiredAttribute class.
        /// </summary>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateRequiredAttribute(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
        {
            if (value == null || value.ToString().Length < 1)
            {
                return false;
            }

            // Otherwise
            return true;
        }
    }
}
