using System;
using System.Reflection;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property is required to be null or empty if another property is not null or empty. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]
    public sealed class ValidateRequiredNullOrEmptyIfNotNullOrEmptyAttribute : EvilBaseAttribute
    {
        private string _propertyName;

        /// <summary>
        /// Gets the name of the property to compare.
        /// </summary>
        public string PropertyName
        {
            get { return _propertyName; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateRequiredNullOrEmptyIfNotNullOrEmptyAttribute class.
        /// </summary>
        /// <param name="propertyName">The name of the property to compare.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateRequiredNullOrEmptyIfNotNullOrEmptyAttribute(string propertyName, string message)
            : base(message)
        {
            _propertyName = propertyName;
        }

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
        {
            object propValue = base.GetPropertyValue(this.PropertyName, entity);

            if (propValue == null || propValue.ToString().Length == 0)
            {
                return true;
            }
            else
            {
                // Check if the value is not populated
                if (value == null || value.ToString().Length == 0)
                {
                    return true;
                }
            }

            // Otherwise
            return false;
        }
    }
}
