using System;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property must be a whole number. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public sealed class ValidateWholeNumberAttribute : EvilBaseAttribute
    {
        /// <summary>
        /// Initializes a new instance of the ValidateWholeNumberAttribute class.
        /// </summary>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateWholeNumberAttribute(string message)
            : base(message)
		{
		}

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
		{
            long temp;

            if (long.TryParse(value.ToString(), out temp))
            {
                return true;
            }

            // Otherwise
            return false;
		}
    }
}
